#include <avr/interrupt.h>                     /* Interruptbehandlungsroutinen (für Timerinterrupt) */
#include <avr/sleep.h>


#define WIDTH 12                               /* Breite des Displays - entspricht x*/
#define HEIGHT 10                              /* Höhe des Displays - entsprcht y*/

#define disp_pin0 14
#define disp_pin1 15
#define disp_pin2 16
#define disp_pin3 17
#define disp_pin4 4
#define disp_pin5 5
#define disp_pin6 6
#define disp_pin7 7
#define disp_pin8 8
#define disp_pin9 9
#define disp_pin_data 12
#define disp_pin_clock 11
#define disp_pin_strobe 10
#define INT0pin 2 //fragt Münzschlitz ab und gibt Strom für Potis

#define frame_slowness 0x8; //Wie lange gewartet wird bis nächste Spalte gezeichnet wird. - Wert nicht zu klein machen!

volatile uint8_t leds[WIDTH][HEIGHT]; //Höhe = die 10 (der 16) Bits
volatile uint8_t aktZeile; //Spalten-Abtast-Counter
volatile uint8_t brightness_cnt;

volatile uint16_t poti_a;
volatile uint16_t poti_b;
volatile uint8_t adc_switch;

volatile uint16_t permanent_cnt; //Hilft bei Zeit-Funktionen

//alte Position speichern um Bewegung zu erkennen
uint8_t pa_alt;
uint8_t pb_alt;

void one_reg();
void null_reg();
void empty_reg();
void fill_reg(); //=Bildschirm löschen weil Negativ-Schaltung mit Register
void set_disp_ports();
void clr_disp_ports();
void adc_init();
void component_inits();
void power_off();
void power_on();

ISR(ADC_vect);
ISR(TIMER2_COMP_vect);
ISR(INT0_vect);


void setup()
{
  cli();
  
  aktZeile=0;
  brightness_cnt=0;
  permanent_cnt=0;

  //Bildchen init/generieren
  for(int x=0;x<WIDTH;x++){
    for(int y=0;y<HEIGHT;y++){
      leds[x][y]=0;
    }
  }
  
  /*---------------------------------------------------
   * Ports konfigurieren (Ein-/Ausgänge)
   *---------------------------------------------------*/
  
  pinMode(disp_pin0,OUTPUT);
  pinMode(disp_pin1,OUTPUT);
  pinMode(disp_pin2,OUTPUT);
  pinMode(disp_pin3,OUTPUT);
  pinMode(disp_pin4,OUTPUT);
  pinMode(disp_pin5,OUTPUT);
  pinMode(disp_pin6,OUTPUT);
  pinMode(disp_pin7,OUTPUT);
  pinMode(disp_pin8,OUTPUT);
  pinMode(disp_pin9,OUTPUT);
  pinMode(disp_pin_data,OUTPUT);
  pinMode(disp_pin_clock,OUTPUT);
  pinMode(disp_pin_strobe,OUTPUT);
  pinMode(INT0pin,OUTPUT);
  
  digitalWrite(INT0pin,LOW);
  
  fill_reg(); //mit HIGH um LEDs auszuschalten

  component_inits();
  
  set_sleep_mode(SLEEP_MODE_ADC);
  
  sei();
  //bringt vor allem Performance wenn man es einfach dauer-an lässt:
  digitalWrite(disp_pin_strobe, HIGH);
  
  /*
  //alte Position speichern um Bewegung zu erkennen
  uint8_t pa_alt;
  uint8_t pb_alt;
  */
}
void loop(){
    //liest Potis aus
    if((ADCSRA & _BV(ADSC)) == 0){ //_bv = bit_value
          sleep_mode();
    }
    
    //berechnet Poti-Wert auf Display-Maße
    uint8_t pa=poti_a/84;
    uint8_t pb=poti_b/96;
    if(pa>=WIDTH) pa=WIDTH-1;
    if(pb>=HEIGHT) pb=HEIGHT-1;
    int t=leds[WIDTH-pa-1][pb];//Helligkeitswert zwischenspeichern
    leds[WIDTH-pa-1][pb]=5;//Helligkeit auf ganz hoch -> Positionscursor
    if((pa==pa_alt)&&(pb==pb_alt)){
      if(permanent_cnt>1000){
        if(t<5) t++;
        //prüft ob ausgeschaltet wird
        if((poti_a>1010)&&(poti_b>1010)){
          power_off();
          power_on();
          t=0;
        }
        permanent_cnt=0;
      }
    } else permanent_cnt=0;
    pa_alt=pa;
    pb_alt=pb;
    
    delay(1);
    leds[WIDTH-pa-1][pb]=t;
    
}

ISR(TIMER2_COMP_vect){
		cli();
		
		if(aktZeile==0){
			clr_disp_ports();	//anti-flirr ;-)
			null_reg();
			set_disp_ports();
			aktZeile++;
		} else {
			clr_disp_ports();        //anti-flirr
			one_reg();
			set_disp_ports();
			aktZeile++;
			if(aktZeile>=WIDTH){
				aktZeile=0;
                                brightness_cnt++;
                                if(brightness_cnt>5) brightness_cnt=0;
			}
		}
                permanent_cnt++;
		sei();
}

void one_reg()
{
  digitalWrite(disp_pin_data, HIGH);         /* Eins schicken (PB4 = HIGH) */
  
  //digitalWrite(disp_pin_strobe, HIGH);
  digitalWrite(disp_pin_clock, HIGH);
  digitalWrite(disp_pin_clock, LOW);
  //digitalWrite(disp_pin_strobe, LOW);
  
  digitalWrite(disp_pin_data, LOW);        /* Data wieder auf LOW rücksetzen (PB4 = LOW) */ 
}

void null_reg()
{
  digitalWrite(disp_pin_data, LOW);
  
  //digitalWrite(disp_pin_strobe, HIGH);
  digitalWrite(disp_pin_clock, HIGH);
  digitalWrite(disp_pin_clock, LOW);
  //digitalWrite(disp_pin_strobe, LOW);
}

//füllt mit LOW
void empty_register()
{
  for(int i=0;i<16;i++){
    null_reg();
  }
}

//füllt mit HIGH
void fill_reg()
{
  for(int i=0;i<16;i++){
    one_reg();
  }
}

void set_disp_ports()
{
  //-----------------------------------------  		
  if(leds[aktZeile][0]<=brightness_cnt) digitalWrite(disp_pin0,LOW); else digitalWrite(disp_pin0,HIGH);
  if(leds[aktZeile][1]<=brightness_cnt) digitalWrite(disp_pin1,LOW); else digitalWrite(disp_pin1,HIGH);
  if(leds[aktZeile][2]<=brightness_cnt) digitalWrite(disp_pin2,LOW); else digitalWrite(disp_pin2,HIGH);
  if(leds[aktZeile][3]<=brightness_cnt) digitalWrite(disp_pin3,LOW); else digitalWrite(disp_pin3,HIGH);
  if(leds[aktZeile][4]<=brightness_cnt) digitalWrite(disp_pin4,LOW); else digitalWrite(disp_pin4,HIGH);
  if(leds[aktZeile][5]<=brightness_cnt) digitalWrite(disp_pin5,LOW); else digitalWrite(disp_pin5,HIGH);
  if(leds[aktZeile][6]<=brightness_cnt) digitalWrite(disp_pin6,LOW); else digitalWrite(disp_pin6,HIGH); 
  if(leds[aktZeile][7]<=brightness_cnt) digitalWrite(disp_pin7,LOW); else digitalWrite(disp_pin7,HIGH);  
  if(leds[aktZeile][8]<=brightness_cnt) digitalWrite(disp_pin8,LOW); else digitalWrite(disp_pin8,HIGH); 
  if(leds[aktZeile][9]<=brightness_cnt) digitalWrite(disp_pin9,LOW); else digitalWrite(disp_pin9,HIGH);
  //-----------------------------------------
}

void clr_disp_ports()
{
//-----------------------------------------  		
  digitalWrite(disp_pin0,LOW);
  digitalWrite(disp_pin1,LOW);
  digitalWrite(disp_pin2,LOW);
  digitalWrite(disp_pin3,LOW);
  digitalWrite(disp_pin4,LOW);
  digitalWrite(disp_pin5,LOW);
  digitalWrite(disp_pin6,LOW);
  digitalWrite(disp_pin7,LOW);
  digitalWrite(disp_pin8,LOW);
  digitalWrite(disp_pin9,LOW);
  //-----------------------------------------
  //Hier könnte das Register auch noch gelöscht werden...
}

ISR(ADC_vect){
	cli();
	
	if(adc_switch==6){
		poti_b=ADCW;
		ADMUX =0b01000111;
		adc_switch=7;
	} else {
		poti_a=ADCW;
		ADMUX =0b01000110;
		adc_switch=6;
	}

	sei();
}

void adc_init(){
	adc_switch=6;	//adc6
	
	//starte ADC6+ADC7

	//ref-spannung auf AVCC, adjust left für 10bit-ergebnis, am ende 0110=ADC6 0111=ADC7
	//welcher, kann während umwandlung verändert werden und bleibt bis sie abgeschlossen ist
	ADMUX =0b01000110; //ADC6
	ADCSRA = _BV(ADEN) | _BV(ADIE) | _BV(ADPS2) | _BV(ADPS1) | _BV(ADPS0);
        //ADCSRA=0b11101111;
}

void component_inits(){
        aktZeile=0;
        brightness_cnt=0;
        permanent_cnt=0;
      
        //Bildchen init/generieren
        for(int x=0;x<WIDTH;x++){
          for(int y=0;y<HEIGHT;y++){
            leds[x][y]=0;
          }
        }
        
	// Timer0 ist leider schon in Verwendung, wenn nicht die entsprechenden Abschnitte in wiring.c und wiring.h auskommentiert werden
        // mit der folge, dass die delay-Routinen aus der avr-gcc-lib übernommen werden müssen
        // er muss aber trotzdem init()ialisiert werden, sonst wacht er aus dem Schlaf nicht mehr auf (diese Routine wird nach dem
        // Einschalten ebenfalls aufgerufen)
        
        init();
        //Timer 2 (8-Bit) für Multiplexing der LEDs
	TCCR2 = (1<<CS22)|(1<<CS21)|(1<<WGM21);	                // 1/256 Vorteiler, CTC
	OCR2= frame_slowness;					// einfach ein init-wert
	TIMSK |= (1<<OCIE2);				        // compare-irq enabled

        adc_init(); //ADC on
        
}

void power_off()
{
                //sleep einstellen
	        set_sleep_mode(SLEEP_MODE_PWR_DOWN);

		ADCSRA=0; //ADC complete conversion
		ADCSRA=0; //ADC off

		//timer deaktivieren
		TCCR0=0;
		TCCR1B=0;
		TCCR2=0;

		fill_reg();
		clr_disp_ports();


		//irgendwie gehts auch ohne folgende Zeile *schulterzuck*
                pinMode(INT0pin,INPUT);
                digitalWrite(INT0pin,HIGH);  //PullUP = Spg von Potis nehmen und INT0 sensitiv für gnd-schalter machen
		//int0 aktivieren (auf low-level sensitiv)
		MCUCR &= ~(0x3);      		// levelgesteuerter Interrupt an INT0
		GICR |= (1 << INT0);            // externen Interrupt freigeben
		//schlaf bis int0 (schalter) aufweckt
		sleep_mode();
}

void power_on()
{
                pinMode(INT0pin,OUTPUT);        // 5V wieder deaktivieren (->Potis aktiviert)
                digitalWrite(INT0pin,LOW);
                
                digitalWrite(disp_pin_strobe, HIGH); //damit reg die daten übernimmt
                
                set_sleep_mode(SLEEP_MODE_ADC);
		component_inits();
                
                sei();
}

ISR(INT0_vect) {
	GICR &= ~(1 << INT0);            // externen Interrupt sperren
}
